package org.rcisoft.business.memTraffic.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.entity.MemLikeDTO;
import org.rcisoft.business.memTraffic.bean.MemTrafficRedisBean;
import org.rcisoft.business.sysScheduledTaskLog.entity.SysScheduledTaskLog;
import org.rcisoft.business.sysScheduledTaskLog.service.SysScheduledTaskLogService;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.business.memTraffic.dao.MemTrafficRepository;
import org.rcisoft.business.memTraffic.entity.MemTraffic;
import org.rcisoft.business.memTraffic.service.MemTrafficService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.math.BigInteger;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2025年2月13日 上午10:00:53.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemTrafficServiceImpl extends ServiceImpl<MemTrafficRepository,MemTraffic>  implements MemTrafficService {



    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private CyRedisService cyRedisService;
    @Autowired
    private CyRedisService cyRedisServiceImpl;
    @Autowired
    private SysScheduledTaskLogService sysScheduledTaskLogServiceImpl;

    /**
    * 保存 访问量表
    * @param memTraffic
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MemTraffic memTraffic){

        // 获取当前日期
        LocalDate today = LocalDate.now();
        // 定义日期时间格式
        DateTimeFormatter formatterNow = DateTimeFormatter.ofPattern("MMdd");
        String date = today.format(formatterNow);

        BigInteger memberId = memTraffic.getMemberId();
        Object isFirst = cyRedisServiceImpl.get(MemTrafficRedisBean.USER_MEMTRAFFIC + date +':'+ memberId);

        if (isFirst == null){
            // 定义日期时间格式
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            // 最大时间 和 最小时间
            String createDate = today.format(formatter);

            memTraffic.setMemTraffic(BigInteger.valueOf(0));
            memTraffic.setBeginTime(createDate);

            baseMapper.insertMemTraffic(memTraffic);

            cyRedisServiceImpl.set(MemTrafficRedisBean.USER_MEMTRAFFIC + date +':'+ memberId,1L, Long.valueOf(48 * 60 * 60));
        } else {

            this.redisTemplate.opsForValue().increment(MemTrafficRedisBean.USER_MEMTRAFFIC + date +':'+ memberId, 1L);
        }
        return new CyPersistModel(1);
    }

    /**
     * 用户访问量定时任务
     */
    public void memTrafficSchedule() {
        log.info("用户访问数据定时任务执行中...");
        //获取当前时间
        // 获取当前日期
        LocalDate today = LocalDate.now();
        // 获取今天的最小时间（00:00:00）
        LocalDateTime startOfDay = today.atStartOfDay();
        // 获取今天的最大时间（23:59:59）
        LocalDateTime endOfDay = today.atTime(LocalTime.MAX);
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 最大时间 和 最小时间
        String beginTime = startOfDay.format(formatter);
        String endTime = endOfDay.format(formatter);

        //将redis中存储的点赞数同步到opm_user_like表中
        Set<String> keys = redisTemplate.keys(MemTrafficRedisBean.USER_MEMTRAFFIC + "*");
        try {
            if (keys != null && !keys.isEmpty()) {
                for (String key : keys) {
                    // 提取 userId
                    int lastIndex = key.lastIndexOf(':');
                    BigInteger memberId;
                    memberId = BigInteger.valueOf(Long.parseLong(key.substring(lastIndex + 1)));
                    //获取访问量
                    String value = redisTemplate.opsForValue().get(key);
                    BigInteger count = null;
                    if (value != null) {
                        count = BigInteger.valueOf(Long.parseLong(value));
                    }

                    //将redis中存储的访问量同步到表中
                    MemTraffic memTraffic = new MemTraffic();
                    memTraffic.setMemberId(memberId);
                    memTraffic.setMemTraffic(count);
                    memTraffic.setBeginTime(beginTime);
                    memTraffic.setEndTime(endTime);
                    baseMapper.updateMemTraffic(memTraffic);
                }
            }
            SysScheduledTaskLog sysScheduledTaskLog = new SysScheduledTaskLog();
            sysScheduledTaskLog.setTaskName("用户访问数据定时任务");
            sysScheduledTaskLog.setTaskState("成功");
            sysScheduledTaskLog.setTaskTime("每20分钟执行一次");
            sysScheduledTaskLog.setCreateDate(new Date());
            sysScheduledTaskLogServiceImpl.persist(sysScheduledTaskLog);
        } catch (Exception e){
            SysScheduledTaskLog sysScheduledTaskLog = new SysScheduledTaskLog();
            sysScheduledTaskLog.setTaskName("用户访问数据定时任务");
            sysScheduledTaskLog.setTaskState("失败");
            sysScheduledTaskLog.setTaskTime("每20分钟执行一次");
            sysScheduledTaskLog.setCreateDate(new Date());
            sysScheduledTaskLogServiceImpl.persist(sysScheduledTaskLog);
        }
    }

    /**
    * 删除 访问量表
    * @param memTraffic
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MemTraffic memTraffic){
        int line = baseMapper.realDelete(memTraffic);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            memTraffic.getBusinessId()+"的访问量表信息");
        return new CyPersistModel(line);
    }

    /**
    * 修改 访问量表
    * @param memTraffic
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MemTraffic memTraffic){
        int line = baseMapper.updateById(memTraffic);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memTraffic.getBusinessId()+"的访问量表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 访问量表
    * @param id
    * @return
    */
    @Override
    public MemTraffic findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 访问量表
    * @param memTraffic
    * @return
    */
    @Override
    public IPage<MemTraffic> findAllByPagination(CyPageInfo<MemTraffic> paginationUtility,
        MemTraffic memTraffic){
        return baseMapper.queryMemTrafficsPaged(paginationUtility,memTraffic);

    }

    /**
    * 查询list 访问量表
    * @param memTraffic
    * @return
    */
    @Override
    public List<MemTraffic> findAll(MemTraffic memTraffic){
        return baseMapper.queryMemTraffics(memTraffic);

    }

    /**
    * 导出访问量表
    * @return
    */
    @Override
    public List<MemTraffic> export(MemTraffic memTraffic) {
        List<MemTraffic> memTrafficList = baseMapper.queryMemTraffics(memTraffic);
        return memTrafficList;
    }

}
